

################# # IMPORTS # #################

library(dplyr)
library(lubridate)
library(plotly)
library(scales)
library(tidyverse)
library(viridis)
library(readxl)
library(zoo)

####################################################################################
#### NOTES #########################################################################
####################################################################################

# This script takes in USDA census and survey data from 1990 to 2020 and performs
# cleaning, merging and interpolation to provide a continious data set to be used 
# by the EX-ACT model stored in VTCarbonBudget/data/model


####################################################################################
#### DATA #############################################################
####################################################################################

census = read_csv("data/input/usda_census_1997-2019.csv") %>% 
  filter(Period == 'YEAR', `Domain Category` == 'NOT SPECIFIED', !Value %in% c('(H)','(L)','(D)','(Z)'),
         !is.na(Value)) %>% #Year > 2019 | Year < 1997) %>% 
  mutate(acres = grepl('acres',`Data Item`,ignore.case = TRUE),
         tonnage = grepl('PRODUCTION, MEASURED IN TONS',`Data Item`,ignore.case = TRUE),
         head = grepl('HEAD',`Data Item`,ignore.case = TRUE),
         Value = as.numeric(gsub(",", "", Value)),
         HARVEST_DESC = sub(".*- ", "", `Data Item`),
         CROP_DESC = sub(" -.*", "", `Data Item`)) %>% 
  filter(!is.na(Value))

survey = read_csv("data/input/usda_survey_1990-2021.csv") %>% 
  filter(Period == 'YEAR', `Domain Category` == 'NOT SPECIFIED',
         !Value %in% c('(H)','(L)','(D)','(Z)','(S)','NA','(NA)'), !is.na(Value),
         !grepl('-',Value)) %>% #Year > 2019 | Year < 1997) %>% 
  mutate(acres = grepl('acres',`Data Item`,ignore.case = TRUE),
         tonnage = grepl('PRODUCTION, MEASURED IN TONS',`Data Item`,ignore.case = TRUE),
         head = grepl('HEAD',`Data Item`,ignore.case = TRUE),
         Value = as.numeric(gsub(",", "", Value)),
         HARVEST_DESC = sub(".*- ", "", `Data Item`),
         CROP_DESC = sub(" -.*", "", `Data Item`)) %>% 
  filter(!is.na(Value))

crop_mgmt_raw = read_excel("data/input/cropland_management.xlsx",skip=1)
  

##### VALIDATION #######
crop_mgmt_raw %>% # Management
  gather(tillage,till_pct,`Full tillage`:`No tillage`) %>% 
  ggplot(aes(Year,till_pct,fill=tillage)) +
  geom_area() +
  # facet_wrap(~Crop,ncol = 1)
  # facet_wrap(Crop~.,ncol = 1 ,labeller = labeller(Crop = label_wrap_gen(5))) +
  facet_grid(Crop~.,labeller = label_wrap_gen(width = 2, multi_line = TRUE)) +
  ggtitle("Tillage Management",subtitle = "By crop type from 1990 - 2020") +
  ylab("Percent") +
  scale_y_continuous(breaks = c(0.5,1), labels = scales::percent_format(accuracy = 1)) +
  theme_minimal() +
  scale_fill_viridis(discrete = TRUE) +
  theme(strip.text = element_text(size=6.5))
  ggsave("plots/cropland_tillage_mgmt.png",width = 7,height = 9)

crop_mgmt_raw %>% # Input
  gather(input,input_pct,`Low C input`:`High C input, with manure`) %>% 
  ggplot(aes(Year,input_pct,fill=input)) +
  geom_area() +
  # facet_wrap(~Crop,ncol = 1)
  # facet_wrap(Crop~.,ncol = 1 ,labeller = labeller(Crop = label_wrap_gen(5))) +
  facet_grid(Crop~.,labeller = label_wrap_gen(width = 2, multi_line = TRUE)) +
  ggtitle("Input Management",subtitle = "By crop type from 1990 - 2020") +
  ylab("Percent") +
  scale_y_continuous(breaks = c(0.5,1), labels = scales::percent_format(accuracy = 1)) +
  theme_minimal() +
  scale_fill_viridis(discrete = TRUE) +
  theme(strip.text = element_text(size=6.5))
  ggsave("plots/cropland_input_mgmt.png",width = 7,height = 9)

crop_mgmt_raw %>% # Residue
  gather(residue,residue_pct,`Burned`:`Retained`) %>%
  ggplot(aes(Year,residue_pct,fill=residue)) +
  geom_area() +
  # facet_wrap(~Crop,ncol = 1)
  # facet_wrap(Crop~.,ncol = 1 ,labeller = labeller(Crop = label_wrap_gen(5))) +
  facet_grid(Crop~.,labeller = label_wrap_gen(width = 2, multi_line = TRUE)) +
  ggtitle("Residue Management",subtitle = "By crop type from 1990 - 2020") +
  ylab("Percent") +
  scale_y_continuous(breaks = c(0.5,1), labels = scales::percent_format(accuracy = 1)) +
  theme_minimal() +
  scale_fill_viridis(discrete = TRUE) +
  theme(strip.text = element_text(size=6.5))
  ggsave("plots/cropland_residue_mgmt.png",width = 7,height = 9)

################# # MANIPULATION # #################

crop_mgmt = crop_mgmt_raw %>% 
  select(Crop:`Retained`) %>% 
  gather(tillage,till_pct,`Full tillage`:`No tillage`) %>% 
  gather(input,input_pct,`Low C input`:`High C input, with manure`) %>% 
  gather(residue,residue_pct,`Burned`:`Retained`) %>%
  filter(till_pct > 0, input_pct > 0, residue_pct > 0) %>% 
  mutate(crop_pct = till_pct * input_pct * residue_pct) %>% 
  # filter(Year == 2020, Crop == 'CORN, SILAGE') %>% 
  select(Crop, Year, `EX-ACT Crop`,crop_pct,tillage,input,residue)


crop_list = c('HAYLAGE, (EXCL ALFALFA)',
              'HAY, (EXCL ALFALFA)',
              'CORN, SILAGE',
              'HAYLAGE, ALFALFA',
              'HAY, ALFALFA',
              'CORN, GRAIN',
              'SOYBEANS',
              'CUT CHRISTMAS TREES',
              'VEGETABLE TOTALS, IN THE OPEN',
              'ORCHARDS',
              'SWEET CORN',
              'BERRY TOTALS',
              'RYE',
              'WHEAT, WINTER',
              'BARLEY',
              'OATS')

survey_crops = survey %>% # PUT SURVEY INTO SHORT CROP DATAFRAME
  filter(acres == TRUE, CROP_DESC %in% crop_list) %>% 
  group_by(Program, Year,HARVEST_DESC,CROP_DESC) %>% 
  summarize(Value)

census_crops = census %>% # PUT CENSUS INTO SHORT CROP DATAFRAME
  filter(acres == TRUE, CROP_DESC %in% crop_list) %>% 
  group_by(Program, Year,HARVEST_DESC,CROP_DESC) %>% 
  summarize(Value)

crops_combined = survey_crops %>% # COMBINE AND CLEAN UP
  rbind(census_crops) %>% 
  spread(Program, Value) %>% 
  mutate(Value = if_else(is.na(CENSUS),SURVEY,CENSUS), Program = if_else(is.na(CENSUS),'SURVEY','CENSUS')) %>% 
  group_by(Year,CROP_DESC, Program) %>% 
  summarize(Value = max(Value))

# CONVERT BU TO TONS
bu_to_tons = tribble(
  ~"Commodity", ~"FACTOR",
  "WHEAT", 0.0272155,
  "OATS", 0.0154221, #64.8420,
  "BARLEY", 0.021772,
  "CORN", 0.0254,
  "SOYBEANS",	0.0272155,
  "RYE", 0.02540134 #39.3680
)

crop_production = census %>% # COMBINE AND FIND PRODUCTION -> METRIC TONS
  rbind(survey) %>%
  filter(CROP_DESC %in% crop_list) %>% 
  filter(tonnage == TRUE | grepl("PRODUCTION, MEASURED IN BU",HARVEST_DESC,ignore.case = TRUE)) %>% 
  mutate(tons_or_bu = if_else(grepl("PRODUCTION, MEASURED IN BU",HARVEST_DESC,ignore.case = TRUE),"BU","T")) %>% 
  select(Year, Program, CROP_DESC, Commodity, tons_or_bu, Value) %>% 
  spread(Program, Value) %>% 
  mutate(Value = if_else(is.na(CENSUS),SURVEY,CENSUS), Program = if_else(is.na(CENSUS),'SURVEY','CENSUS')) %>% 
  select(Year, CROP_DESC, Commodity,Program, tons_or_bu, Value) %>% 
  left_join(bu_to_tons, by = c("Commodity")) %>% 
  mutate(metric_tons = case_when(!is.na(FACTOR) & tons_or_bu == "BU" ~ Value * FACTOR,
                                 is.na(FACTOR) & tons_or_bu == "BU" ~ Value * 0.907185,
                                 TRUE ~ Value)) %>% 
  select(CROP_DESC, Program, Year, metric_tons)


####### CROP HARVESTED FILLER #########

list = unique(crops_combined$CROP_DESC)
# Backfill years
backfill = data.frame()
for (i in 1:length(list)){
  temp = data.frame(Year = 1990:2021, 
                    CROP_DESC = list[i], 
                    Value = 0,
                    Program = 'temp')
  
  backfill = rbind(backfill,temp)
}

crops_full = rbind(crops_combined, backfill) %>% 
  group_by(Year,CROP_DESC) %>% 
  summarize(Program = min(Program), Value = max(Value)) %>% 
  mutate(Value = ifelse(Value == 0, NA,Value))

crop_temp = rbind(crops_combined, backfill) %>% 
  group_by(Year,CROP_DESC) %>% 
  summarize(Program = min(Program), Value = max(Value)) %>% 
  mutate(Value = ifelse(Value == 0, NA,Value),
         Program = ifelse(Program == 'temp','INTERPOLATED',Program))

starter = crop_temp %>% 
  filter(!is.na(Value)) %>% 
  group_by(CROP_DESC) %>% 
  summarize(Year = min(Year))

ender = crop_temp %>% 
  filter(!is.na(Value)) %>% 
  group_by(CROP_DESC) %>% 
  summarize(Year = max(Year))

first_value = starter %>% 
  left_join(crop_temp) %>% 
  mutate(Year = min(crop_temp$Year) - 1)

last_value = ender %>% 
  left_join(crop_temp) %>% 
  mutate(Year = max(crop_temp$Year) + 1)

crop_full = first_value %>% 
  rbind(crop_temp) %>% 
  rbind(last_value)

crop = crop_full %>% 
  group_by(CROP_DESC) %>% 
  mutate(Value_int = na.approx(Value)) %>% 
  filter(Year >= min(crop_temp$Year), Year <= max(crop_temp$Year)) %>% 
  mutate(Crop = if_else(CROP_DESC == 'VEGETABLE TOTALS, IN THE OPEN','VEGETABLES',CROP_DESC),
         Value = Value_int) %>% 
  ungroup() %>% 
  select(Crop,Year, Program, Value) %>% 
  filter(!Crop %in% c("ORCHARDS","CUT CHRISTMAS TREES","BERRY TOTALS"))


perennial_crop = crop_full %>% 
  group_by(CROP_DESC) %>% 
  mutate(Value_int = na.approx(Value)) %>% 
  filter(Year >= min(crop_temp$Year), Year <= max(crop_temp$Year)) %>% 
  mutate(Crop = if_else(CROP_DESC == 'VEGETABLE TOTALS, IN THE OPEN','VEGETABLES',CROP_DESC),
         Value = Value_int) %>% 
  ungroup() %>% 
  select(Crop,Year, Program, Value) %>% 
  filter(Crop %in% c("ORCHARDS","CUT CHRISTMAS TREES","BERRY TOTALS"))

######## CROP PRODUCTION FILLER #########

p_list = unique(crop_production$CROP_DESC)
# Backfill years
p_backfill = data.frame()
for (i in 1:length(p_list)){
  temp = data.frame(Year = 1990:2021, 
                    CROP_DESC = p_list[i],
                    Program = 'temp',
                    metric_tons = 0)
  
  p_backfill = rbind(p_backfill,temp)
}

p_crops_full = rbind(crop_production, p_backfill) %>% 
  group_by(Year,CROP_DESC) %>% 
  summarize(Program = min(Program), metric_tons = max(metric_tons)) %>% 
  mutate(metric_tons = ifelse(metric_tons == 0, NA,metric_tons), Crop = CROP_DESC) %>% 
  select(Crop, Year, metric_tons)


yield_crop = crop %>% 
  left_join(p_crops_full, by = c("Year","Crop")) %>% 
  mutate(`t/ha/yr` = metric_tons/(Value*0.404686)) %>% 
  select(-metric_tons, -Value) %>% 
  add_row(Crop = "VEGETABLES", Year = 1989,Program = 'INPUT',`t/ha/yr`= 20) %>% 
  add_row(Crop = "SWEET CORN", Year = 1989,Program = 'INPUT',`t/ha/yr`= 40) 

yield_crop_temp = yield_crop %>% 
  filter(!is.na(`t/ha/yr`))


yield_starter = yield_crop %>%
  filter(!is.na(`t/ha/yr` )) %>%
  group_by(Crop) %>%
  summarize(Year = min(Year))

yield_ender = yield_crop %>%
  filter(!is.na(`t/ha/yr` )) %>%
  group_by(Crop) %>%
  summarize(Year = max(Year))

yield_first_value = yield_starter %>%
  left_join(yield_crop) %>%
  mutate(Year = min(yield_crop$Year) - 1)

yield_last_value = yield_ender %>%
  left_join(yield_crop) %>%
  mutate(Year = max(yield_crop$Year) + 1)

yield_crop_full = yield_first_value %>%
  rbind(yield_crop) %>%
  rbind(yield_last_value)

yield_crop_final = yield_crop_full %>%
  group_by(Crop) %>%
  mutate(`t/ha/yr` = na.approx(`t/ha/yr`)) %>%
  filter(Year >= min(crop$Year), Year < max(crop$Year)) %>%
  # mutate(Crop = if_else(Crop == 'VEGETABLE TOTALS, IN THE OPEN','VEGETABLES',CROP_DESC)) %>%
  ungroup() %>%
  select(Crop, Year, Program, `t/ha/yr`)


####### COMBINE IT ALL!! #########

crop_final = crop_mgmt %>% 
  left_join(yield_crop_final, by = c("Crop","Year")) %>% 
  select(-Program) %>% 
  left_join(crop, by = c("Crop","Year")) %>% 
  mutate(ha = 0.404686 * Value * crop_pct,counter = 1:n()) %>% 
  select(counter, Year, Crop, `EX-ACT Crop`, tillage, input, residue, `t/ha/yr`,ha)


perennial_final = perennial_crop %>% 
  mutate(`EX-ACT Crop` = case_when(Crop == 'CUT CHRISTMAS TREES' ~ 'Agroforestry - default',
                                   Crop == 'ORCHARDS' ~ 'Orchard',
                                   Crop == 'BERRY TOTALS' ~ 'Vine'),
         Tillage = 'No tillage',
         Input = case_when(Crop == 'CUT CHRISTMAS TREES' ~ 'Medium C input',
                           Crop == 'ORCHARDS' ~ 'Medium C input',
                           Crop == 'BERRY TOTALS' ~ 'High C input, with manure'),
         Burning = 'NO',
         yield = "",
         ha = 0.404686 * Value,
         counter = 1:n()) %>% 
  select(counter,Year, Crop, `EX-ACT Crop`, Tillage, Input, Burning, yield, ha)

# WRITE TO FILE:

crop_final %>% 
  write_csv("data/output/cropland_final_for_exact.csv")

perennial_final %>% 
  write_csv("data/output/perennial_final_for_exact.csv")


#############################################################################
###### PLOTS ######################################################
#############################################################################

crop %>%
  ggplot(aes(Year,Value,color = fct_reorder2(Crop, Year, Value))) +
  geom_line()+
  geom_point(aes(shape=Program),size = 2,alpha=.75) +
  # scale_color_viridis(discrete = TRUE) +
  scale_y_continuous(labels = comma) +
  labs(color = "Crop Description") +
  xlab("") +
  ylab("Acres Harvested") +
  theme_classic() +
  ggtitle("Cropland Acreage",subtitle = "Using USDA Census, Survey and Interpolated Values") #+
ggsave("plots/annual_cropland_acreage.png",width = 10, height = 5)


perennial_plot = perennial_crop %>%
  ggplot(aes(Year,Value,color = fct_reorder2(Crop, Year, Value))) +
  geom_line()+
  geom_point(aes(shape=Program),size = 2,alpha=.75) +
  scale_color_viridis(discrete = TRUE) +
  scale_y_continuous(labels = comma, limits = c(0,250000)) +
  labs(color = "Crop Description") +
  xlab("") +
  ylab("Acres Harvested") +
  theme_classic() +
  ggtitle("Perennial Acreage",subtitle = "Using USDA Census and Interpolated Values")
ggsave("plots/perennial_cropland_acreage.png",width = 10, height = 5)

