# Pulling in tCO2-e data from EX-ACT

library(tidyverse)
library(readxl)
library(dplyr)
library(ggdark)
library(plotly)
library(scales)
library(viridis)

# ONE MILLION
one_million = 1000000 # variable to prevent zero issues when used...
ha_to_acres = 2.47105

# LOAD CROPLAND, PERENNIALS, LIVESTOCK, FERTILIZER
ex_act_cropland = read_excel("data/model/EX-ACT_V9.0_VT.xlsm",sheet='Calculations',
                             range = cell_cols("P:X")) %>%
  mutate(acre = ha * ha_to_acres, 
         Crop = case_when(
           `USDA Crop` %in% c("BARLEY","OATS","RYE","WHEAT, WINTER","SOYBEANS") ~ "Minor grains + soy",
           grepl("CORN,",`USDA Crop`,ignore.case = TRUE) ~ "CORN (grain + silage)",
           `USDA Crop` %in% c("VEGETABLES","SWEET CORN") ~ "VEG. (incl. sweet corn)",
           TRUE ~ `USDA Crop`
         )
  )

ex_act_perennials = read_excel("data/model/EX-ACT_V9.0_VT.xlsm",sheet='Calculations',
                               range = cell_cols("AN:AV")) %>%
  mutate(acre = ha * ha_to_acres)

ex_act_livestock = read_excel("data/model/EX-ACT_V9.0_VT.xlsm",sheet='Calculations',
                              range = cell_cols("AA:AK")) %>%
  mutate(GROUP = case_when(
    `USDA Livestock` == 'CATTLE, COWS, MILK' ~ 'CATTLE, DAIRY COWS',
    `USDA Livestock` == 'CATTLE, (EXCL COWS)' ~ 'CATTLE, CALVES',
    `USDA Livestock` == 'CATTLE, COWS, BEEF' ~ 'CATTLE, BEEF',
    `USDA Livestock` == 'CHICKENS, PULLETS, REPLACEMENT' ~ 'CHICKENS, PULLETS',
    grepl('CHICKEN',`USDA Livestock`) ~ `USDA Livestock`,
    `USDA Livestock` == 'SHEEP, INCL LAMBS' ~ 'SHEEP',
    # CROP_DESC == 'CHICKENS, BROILERS' ~ 'CHICKENS, BROILERS',
    # CROP_DESC == 'CHICKENS, PULLETS, REPLACEMENT' ~ 'CHICKENS, PULLETS',
    TRUE ~ `USDA Livestock`),
    subgroup = case_when(
      grepl('CATTLE',GROUP,ignore.case = TRUE) ~ GROUP,
      GROUP %in% c("ALPACAS","LLAMAS","SHEEP") ~ "ALPACAS, LLAMAS, SHEEP",
      grepl("CHICKENS",GROUP,ignore.case = TRUE) ~ "POULTRY",
      GROUP %in% c("CHUKARS","DUCKS","GEESE","GUINEAS","PEAFOWL, HENS & COCKS","PIGEONS & SQUAB",
                   "POULTRY, OTHER","TURKEYS") ~ "POULTRY",
      GROUP %in% c("EQUINE, HORSES & PONIES","EQUINE, MULES & BURROS & DONKEYS") ~ "EQUINE",
      GROUP == "GOATS" ~ "GOATS",
      GROUP == "HOGS" ~ "HOGS",
      TRUE ~ "OTHER"
    )
  )


ex_act_fertilizer = read_excel("data/model/EX-ACT_V9.0_VT.xlsm",sheet='Calculations',
                               range = cell_cols("BP:BY"))

ex_act_grassland = read_excel("data/model/EX-ACT_V9.0_VT.xlsm",sheet='Calculations',
                              range = cell_cols("AY:BM"))

wetlands = read_csv("data/model/wetlands_mtco2-e_output.csv")

#---------------------------------------------------
### MAKE SOME GRAPHS
#---------------------------------------------------

# ORDER IN REPORT:
# WETLANDS
# CROPLANDS ANNUALS
# CROPLANDS PERENNIALS
# LIVESTOCK
# FERTILIZER
# GRASSLANDS

#---------------------------------------------------
### WETLANDS
#---------------------------------------------------

wetlands %>%
  mutate(`mmtCO2-e` = `tCO2-e`/one_million) %>% 
  ggplot(aes(Year,`mmtCO2-e`)) +
  geom_line(color = "#238A8DFF", size=1.5) +
  scale_color_viridis() +
  # ylab(expression(paste("Wetlands & water bodies \n GHG emissions metric tons CO" [2],"-e yr"^{-1},")")))+
  ylab(expression(atop("Wetlands and water bodies", paste("GHG emissions (million metric tons CO" [2], "-e yr"^{-1},")")))) +
  xlab("") +
  theme_classic(base_size = 14) +
  scale_y_continuous(limits = c(-0.012,-0.011), breaks = c(-.011,-.0112,-.0114,-.0116,-.0118,-.012))
# labels = comma)
ggsave("plots/output_wetlands_mmtco2-e.png",width = 10, height = 5)

wetlands %>% 
  filter(Year %in% c(1990,2005,2000)) %>% 
  mutate(`mmtCO2-e` = `tCO2-e`/one_million)


#---------------------------------------------------
### CROPLAND
#---------------------------------------------------
total_crop_emissions = ex_act_cropland %>% 
  group_by(Year) %>% 
  summarise(`mmtCO2-e`= sum(`tCO2-e`)/one_million) %>% 
  mutate(Crop = 'NET')

q_colors =  8 # for no particular reason
v_colors =  viridis(q_colors)
v_colors[1] = "red1"
names(v_colors) = c("NET",
                    "HAYLAGE, ALFALFA", 
                    "HAY, ALFALFA",
                    "Minor grains + soy",
                    "VEG. (incl. sweet corn)",
                    "CORN (grain + silage)",
                    "HAYLAGE, (EXCL ALFALFA)",
                    "HAY, (EXCL ALFALFA)"
)


ex_act_cropland %>% #  CROPLAND ACRES EX-ACT LABELS
  group_by(Crop,Year) %>% 
  summarize(`mmtCO2-e` = sum(`tCO2-e`)/one_million) %>% 
  ggplot(aes(Year, `mmtCO2-e`, color = fct_reorder2(Crop,Year,`mmtCO2-e`)))+ #color = fct_reorder2(grouped, Year, Value))
  geom_line(size = 1.5) +
  geom_line(data = total_crop_emissions, aes(Year,`mmtCO2-e`), size = 1.5, linetype = "twodash")+
  geom_hline(yintercept = 0.0, color = "gray", size = 1.2, alpha = .75) +
  # scale_y_continuous(labels = comma, limits = c(-50000,190000)) +
  scale_color_manual(values = v_colors)+
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Crop Description",
       x = "",
       y = expression(atop("Annual Cropland GHG emissions", paste("(million metric tons CO" [2], "-e yr"^{-1},")")))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_cropland_mmtco2-e.png",width = 10, height = 5)


# CROP CARBON STOCK 
cropland_carbon_stock_ratios = tribble(~"Subgroup",~"Stock_CO2/ha",
                                       "Corn (silage, grain)",	314, # data from VTAAFM 2019 Fert Report
                                       "Hay/haylage",	365,
                                       "Minor grains + soy",	290,
                                       "Veg", 254)

cropland_carbon_stock = ex_act_cropland %>% 
  mutate(Subgroup = case_when(
    Crop == "VEG. (incl. sweet corn)" ~ "Veg",
    Crop == "CORN (grain + silage)"  ~  "Corn (silage, grain)",
    Crop == "Minor grains + soy"   ~  "Minor grains + soy"   ,
    grepl("HAY",Crop,ignore.case = TRUE) ~ "Hay/haylage"          
  )) %>% 
  group_by(Crop, Subgroup) %>% 
  left_join(cropland_carbon_stock_ratios) %>% 
  mutate(Stock = (ha * `Stock_CO2/ha`)/one_million)

net_cropland_carbon_stock = cropland_carbon_stock %>% 
  group_by(Year) %>% 
  summarize(Stock = sum(Stock)) %>% 
  mutate(Subgroup = 'NET')

q_colors =  5 # for no particular reason
v_colors =  viridis(q_colors)
v_colors[1] = "red1"
names(v_colors) = c("NET",
                    "Hay/haylage", 
                    "Corn (silage, grain)",
                    "Minor grains + soy",
                    "Veg"
)

cropland_carbon_stock %>% 
  group_by(Subgroup, Year) %>% 
  summarize(Stock = sum(Stock)) %>% 
  ggplot(aes(Year, Stock, color = fct_reorder2(Subgroup,Year,Stock)))+ #color = fct_reorder2(grouped, Year, Value))
  geom_line(size = 1.5) +
  geom_line(data = net_cropland_carbon_stock, aes(Year,Stock), size = 1.5, linetype = "twodash")+
  scale_y_continuous(labels = comma, breaks = c(0,20,40,60,80,100)) +
  scale_color_manual(values = v_colors)+
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Crop Description") +
  # ylab("tCO2-e")+
  ylab(expression(paste("Carbon stock (MMT CO" [2], "-e)"))) +
  xlab("") +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_cropland_stock.png",width = 10, height = 5)


#---------------------------------------------------
### PERENNIALS
#---------------------------------------------------

# PERENNIALS
perennials_total = ex_act_perennials %>% 
  group_by(Year) %>% 
  summarize(`tCO2-e` = sum(`tCO2-e`)/one_million) %>% 
  mutate(Crop = 'NET')

q_colors =  4 # for no particular reason
v_colors =  viridis(q_colors)
v_colors[2] = "red1"
names(v_colors) = c("ORCHARDS","NET","CUT CHRISTMAS TREES","BERRY TOTALS")

ex_act_perennials %>% #  PERENNIALS tCO2-e
  group_by(Crop,Year) %>% 
  summarize(`tCO2-e` = sum(`tCO2-e`)/one_million) %>% 
  # full_join(perennials_total) %>% 
  ggplot(aes(Year,`tCO2-e`,color = fct_reorder2(Crop,Year,`tCO2-e`)))+ 
  geom_line(size=1.5)+
  geom_line(data = perennials_total, aes(Year,`tCO2-e`), size = 1.5, linetype = "twodash")+
  geom_hline(yintercept = 0.0, color = "gray", size = 1.2, alpha = .75) +
  scale_y_continuous(labels = comma) +
  # scale_color_viridis(discrete = TRUE) +
  scale_color_manual(values = v_colors)+
  labs(color = "Crop Description") +
  # labs(y = expression("GHG Emissions (metric tons CO[2]-ery^-1"))+
  labs(x = "",
       y = expression(atop("Perennial Cropland GHG emissions", paste("(million metric tons CO" [2], "-e yr"^{-1},")")))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_perennial_mmtco2-e.png",width = 10, height = 5)


combined_cropland = total_crop_emissions %>% 
  rename(crop_co2 = `mmtCO2-e`) %>% 
  select(Year,crop_co2) %>% 
  left_join(perennials_total) %>% 
  transmute(Year, `mmtCO2-e` = crop_co2 + `tCO2-e`, Crop = 'CROPLANDS')


#---------------------------------------------------
### LIVESTOCK
#---------------------------------------------------

ex_act_livestock_total = ex_act_livestock %>% 
  group_by(Year) %>% 
  summarize(mmtCO2 = sum(`tCO2-e`/one_million)) %>% 
  mutate(Livestock = "NET")

q_colors =  9
v_colors =  viridis(q_colors,direction = -1)
v_colors = c("red1",v_colors)
names(v_colors) = c("NET","CATTLE, DAIRY COWS", 
                    "CATTLE, CALVES",
                    "CATTLE, BEEF",
                    "ALPACAS, LLAMAS, SHEEP", 
                    "POULTRY", "EQUINE", "GOATS", "HOGS","OTHER")

ex_act_livestock %>% 
  select(Year,subgroup,`tCO2-e`) %>% 
  rename("Livestock"=subgroup) %>% 
  # filter(Livestock != "OTHER") %>% 
  group_by(Year,Livestock) %>% 
  summarize(mmtCO2 = sum(`tCO2-e`)/one_million) %>%
  ggplot(aes(Year, mmtCO2, color = fct_reorder2(Livestock,Year,mmtCO2))) +
  geom_line(size = 1.5) +
  geom_line(data = ex_act_livestock_total, aes(Year,mmtCO2), size = 1.5, linetype = "twodash")+
  scale_color_manual(values = v_colors)+
  scale_y_continuous(limits = c(0,0.6))+
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Livestock",
       x = "",
       y = expression(atop("Livestock GHG emissions", paste("(million metric tons CO" [2], "-e yr"^{-1},")")))) +
  #   
  # y=expression(paste("Fertlizer GHG Emissions (million metric tons CO"[2],"-e yr"^{-1},')'))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_livestock_mmtco2-e.png",width = 10, height = 5)


#---------------------------------------------------
### FERTILIZERS
#---------------------------------------------------

total_fertilizer = ex_act_fertilizer %>%
  transmute(Year, mmtCO2 = `Total tCO2-e Field`/one_million) %>% 
  mutate(Fertilizer = "NET")


q_colors =  3
v_colors =  viridis(q_colors)
v_colors[1] = "red1"
names(v_colors) = c("NET","N-synthetic","Limestone")

ex_act_fertilizer %>% 
  select(Year, `tCO2-e N`,`tCO2-e limestone`) %>% 
  rename("N-synthetic"=`tCO2-e N`, "Limestone"=`tCO2-e limestone`) %>% 
  gather(Fertilizer, tCO2, "N-synthetic":"Limestone") %>% 
  mutate(mmtCO2 = tCO2/one_million) %>% 
  ggplot(aes(Year, mmtCO2, color = fct_reorder2(Fertilizer,Year,mmtCO2))) +
  geom_line(size = 1.5) +
  geom_line(data = total_fertilizer, aes(Year,mmtCO2), size = 1.5, linetype = "twodash")+
  scale_color_manual(values = v_colors)+
  scale_y_continuous(limits = c(0,0.02))+
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Fertilizer",
       x = "",
       y = expression(atop("Fertilizer GHG emissions", paste("(million metric tons CO" [2], "-e yr"^{-1},")")))) +
  #   
  # y=expression(paste("Fertlizer GHG Emissions (million metric tons CO"[2],"-e yr"^{-1},')'))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_fertlizer_mmtco2-e.png",width = 10, height = 5)

#---------------------------------------------------
### GRASSLAND
#---------------------------------------------------

total_grassland = ex_act_grassland %>%
  transmute(Year, mmtCO2 = `Net tCO2-e`/one_million, acres = `Total Ha` * ha_to_acres) %>% 
  mutate(Grassland = "NET")

q_colors =  4
v_colors =  viridis(q_colors)
v_colors[1] = "red1"
names(v_colors) = c("NET","Non-degraded","Improved grassland","Severely degraded")

ex_act_grassland %>% 
  select(Year, `Severely degraded`,`Non-degraded`,`Improved grassland`) %>% 
  gather(Grassland, ha, `Severely degraded`:`Improved grassland`) %>% 
  mutate(acres = ha*ha_to_acres) %>% 
  ggplot(aes(Year, acres, color = fct_reorder2(Grassland,Year,acres))) +
  geom_line(size = 1.5) +
  geom_line(data = total_grassland, aes(Year,acres), size = 1.5, linetype = "twodash")+
  scale_color_manual(values = v_colors)+
  scale_y_continuous(labels = comma, limits = c(0,400000)) +
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Grassland",
       x = "",
       y = "Grassland and shrubland (acres)") +
  # y=expression(paste("Fertlizer GHG Emissions (million metric tons CO"[2],"-e yr"^{-1},')'))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_grassland_acres.png",width = 10, height = 5)

ex_act_grassland %>% 
  select(Year, `Severely degraded tCO2-e`,`Non-degraded tCO2-e`,`Improved grassland tCO2-e`) %>% 
  # rename("Severely degraded"=`Severely degraded tCO2-e`,
  #        "Non-degraded" = `Non-degraded tCO2-e`,
  #        "Improved grassland" = `Improved grassland tCO2-e`) + 
  gather(Grassland, tCO2, `Severely degraded tCO2-e`:`Improved grassland tCO2-e`) %>%
  mutate(mmtCO2 = tCO2/one_million,
         Grassland = case_when(Grassland == 'Severely degraded tCO2-e' ~ 'Severely degraded',
                               Grassland == 'Non-degraded tCO2-e' ~ 'Non-degraded',
                               Grassland == 'Improved grassland tCO2-e' ~ 'Improved grassland')) %>% 
  ggplot(aes(Year, mmtCO2, color = fct_reorder2(Grassland,Year,mmtCO2))) +
  geom_line(size = 1.5) +
  geom_line(data = total_grassland, aes(Year,mmtCO2), size = 1.5, linetype = "twodash")+
  scale_color_manual(values = v_colors)+
  scale_y_continuous(labels = comma, limits = c(-0.025,.075)) +
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Grassland",
       x = "",
       y = expression(atop("Grassland and shrublad GHG emissions", paste("(million metric tons CO" [2], "-e yr"^{-1},")")))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_grassland_mmtco2-e.png",width = 10, height = 5)


#---------------------------------------------------
### COMBINED: LIVESTOCK, FERTILIZER, CROPLAND
#---------------------------------------------------

total_fertilizer = total_fertilizer %>% 
  mutate(Type = 'FERTILIZER') %>% 
  select(Year,mmtCO2,Type)

ex_act_livestock_total = ex_act_livestock_total %>% 
  mutate(Type = 'LIVESTOCK') %>% 
  select(Year,mmtCO2,Type)

combined_cropland = combined_cropland %>% 
  mutate(Type = 'CROPLAND', mmtCO2 = `mmtCO2-e`) %>% 
  select(Year,mmtCO2,Type)

Total_Ag = rbind(ex_act_livestock_total,total_fertilizer,combined_cropland) %>% 
  group_by(Year) %>% 
  summarize(mmtCO2 = sum(mmtCO2)) %>% 
  mutate(Type = "NET")

# q_colors =  3
v_colors =  c("red1","#90548bff","#55C667FF","#FDE725FF") #viridis(q_colors,direction=-1)
# v_colors = c("red1",v_colors)
names(v_colors) = c("NET","LIVESTOCK","CROPLAND","FERTILIZER")

rbind(total_fertilizer,ex_act_livestock_total,combined_cropland) %>% 
  ggplot(aes(Year, mmtCO2, color = fct_reorder2(Type,Year,mmtCO2))) +
  geom_line(size = 1.5) +
  geom_line(data = Total_Ag, aes(Year,mmtCO2), size = 1.5, linetype = "twodash")+
  scale_color_manual(values = v_colors)+
  scale_y_continuous(labels = comma, limits = c(0,.8)) +
  # scale_color_viridis(discrete = TRUE) +
  labs(color = "Description",
       x = "",
       y = expression(atop("GHG emissions from all agriculture", paste("(million metric tons CO" [2], "-e yr"^{-1},")")))) +
  theme_classic() +
  theme_classic(base_size = 14)
ggsave("plots/ex_act_agriculture_mmtco2-e.png",width = 10, height = 5)


cattle_co2 = ex_act_livestock %>% 
  filter(subgroup %in% c("CATTLE, DAIRY COWS", 
                         "CATTLE, CALVES",
                         "CATTLE, BEEF"),
         Year == 2020) %>% 
  group_by(subgroup) %>% 
  summarise(co2_total = sum(`tCO2-e`))

cattle_mm = ex_act_livestock %>% 
  filter(subgroup %in% c("CATTLE, DAIRY COWS", 
                          "CATTLE, CALVES",
                          "CATTLE, BEEF"),
         Year == 2020) %>% 
  group_by(subgroup,`Complementary manure management system`) %>% 
  summarize(head = sum(Head), co2 = sum(`tCO2-e`)) %>% 
  left_join(cattle_co2, by = c("subgroup")) %>% 
  mutate(co2_per_head = co2/head, co2_pct = co2/co2_total)


Total_Ag_2020 = rbind(ex_act_livestock_total,total_fertilizer,combined_cropland) %>% 
  filter(Year == 2020) %>% 
  mutate(pct_co2 = mmtCO2/sum(mmtCO2))


