
################# # IMPORTS # #################

library(dplyr)
library(googlesheets4)
library(lubridate)
library(plotly)
library(scales)
library(tidyverse)
library(viridis)
library(zoo)

####################################################################################
#### NOTES #########################################################################
####################################################################################

# This script takes in USDA census and survey data from 1990 to 2020 and performs
# cleaning, merging and interpolation to provide a continious data set to be used 
# by the EX-ACT model stored in VTCarbonBudget/data/model

####################################################################################
#### DATA ##########################################################################
####################################################################################

census_animals = read_csv("data/input/usda_vt_livestock_inventory_1997-2017.csv") %>% 
  filter(`Domain Category` == 'NOT SPECIFIED', !Value %in% c('(H)','(L)','(D)','(Z)'),
         !is.na(Value)) %>% #Year > 2019 | Year < 1997) %>% 
  mutate(acres = grepl('acres',`Data Item`,ignore.case = TRUE),
         tonnage = grepl('PRODUCTION, MEASURED IN TONS',`Data Item`,ignore.case = TRUE),
         head = grepl('HEAD',`Data Item`,ignore.case = TRUE),
         Value = as.numeric(gsub(",", "", Value)),
         HARVEST_DESC = sub(".*- ", "", `Data Item`),
         CROP_DESC = sub(" -.*", "", `Data Item`)) %>% 
  filter(!is.na(Value))

survey = read_csv("data/input/usda_survey_1990-2021.csv") %>% 
  filter(Period == 'YEAR', `Domain Category` == 'NOT SPECIFIED',
         !Value %in% c('(H)','(L)','(D)','(Z)','(S)','NA','(NA)'), !is.na(Value),
         !grepl('-',Value)) %>% #Year > 2019 | Year < 1997) %>% 
  mutate(acres = grepl('acres',`Data Item`,ignore.case = TRUE),
         tonnage = grepl('PRODUCTION, MEASURED IN TONS',`Data Item`,ignore.case = TRUE),
         head = grepl('HEAD',`Data Item`,ignore.case = TRUE),
         Value = as.numeric(gsub(",", "", Value)),
         HARVEST_DESC = sub(".*- ", "", `Data Item`),
         CROP_DESC = sub(" -.*", "", `Data Item`)) %>% 
  filter(!is.na(Value))


livestock_mgmt_raw = read_excel("data/input/livestock_management.xlsx", skip = 1) %>% 
  select(Livestock:Total)


##### VALIDATION #######

livestock_mgmt_raw %>% # Management
  filter(Livestock %in% c('CATTLE, COWS, MILK',
                          'CATTLE, (EXCL COWS)',
                          'CATTLE, COWS, BEEF')) %>%
  mutate(Livestock = case_when(
    Livestock == 'CATTLE, COWS, MILK' ~ 'DAIRY',
    Livestock == 'CATTLE, (EXCL COWS)' ~ 'CALVES/ HEIFERS',
    Livestock == 'CATTLE, COWS, BEEF' ~ 'BEEF'
  )) %>% 
  gather(manure_mgmt,mgmt_pct,`Daily spread`:`Pasture/Range/Paddock`) %>% 
  filter(manure_mgmt != 'Dry lot') %>% 
  ggplot(aes(Year,mgmt_pct,fill=manure_mgmt)) +
  geom_area() +
  # facet_wrap(~Crop,ncol = 1)
  # facet_wrap(Crop~.,ncol = 1 ,labeller = labeller(Crop = label_wrap_gen(5))) +
  facet_grid(Livestock~.,labeller = label_wrap_gen(width = 2, multi_line = TRUE)) +
  ggtitle("Cattle Manure Management",subtitle = "By livestock type from 1990 - 2020") +
  ylab("Percent") +
  labs(fill = "Manure Management") +
  scale_y_continuous(breaks = c(0.5,1), labels = scales::percent_format(accuracy = 1)) +
  theme_minimal() +
  scale_fill_viridis(discrete = TRUE) +
  theme(strip.text = element_text(size=10))
ggsave("plots/cattle_manure_mgmt.png",width = 10,height = 5)


livestock_mgmt_raw %>% # Management
  filter(!Livestock %in% c('CATTLE, COWS, MILK',
                           'CATTLE, (EXCL COWS)',
                           'CATTLE, COWS, BEEF')) %>%
  gather(manure_mgmt,mgmt_pct,`Daily spread`:`Pasture/Range/Paddock`) %>% 
  ggplot(aes(Year,mgmt_pct,fill=manure_mgmt)) +
  geom_area() +
  # facet_wrap(~Crop,ncol = 1)
  # facet_wrap(Crop~.,ncol = 1 ,labeller = labeller(Crop = label_wrap_gen(5))) +
  facet_grid(Livestock~.,labeller = label_wrap_gen(width = 2, multi_line = TRUE)) +
  ggtitle("Manure Management",subtitle = "By livestock type from 1990 - 2020") +
  ylab("Percent") +
  labs(fill = "Other Livestock Manure Management") +
  scale_y_continuous(breaks = c(0.5,1), labels = scales::percent_format(accuracy = 1)) +
  theme_minimal() +
  scale_fill_viridis(discrete = TRUE) +
  theme(strip.text = element_text(size=10))
ggsave("plots/other_manure_mgmt.png",width = 9,height = 5)



# MANIPULATION

livestock_mgmt = livestock_mgmt_raw %>% 
  gather(manure_mgmt,mgmt_pct,`Daily spread`:`Pasture/Range/Paddock`) %>% 
  filter(mgmt_pct > 0) %>% 
  mutate(Productivity = 'High-productivity',
         `tCO2-e` = '',
         EF = case_when(Livestock == 'CATTLE, (EXCL COWS)' ~ '50',
                        Livestock == 'CHICKENS, PULLETS, REPLACEMENT' ~ '0',
                        Livestock == 'RABBITS, LIVE' ~ '0',
                        TRUE ~ ''),
         PRP_pct = case_when(
           Livestock == 'CATTLE, (EXCL COWS)' ~ '30%',
           Livestock == 'CHICKENS, PULLETS, REPLACEMENT' ~ '0%',
           Livestock == 'RABBITS, LIVE' ~ '0%',
           TRUE ~ ''
         ),
         PRP_emissions = case_when(
           Livestock == 'CATTLE, (EXCL COWS)' ~ '15%',
           Livestock == 'CHICKENS, PULLETS, REPLACEMENT' ~ '0%',
           Livestock == 'RABBITS, LIVE' ~ '0%',
           TRUE ~ ''
         ))

animal_list = c('CATTLE, COWS, MILK',
                'CATTLE, (EXCL COWS)',
                'CATTLE, COWS, BEEF',
                'CHICKENS, LAYERS',
                'CHICKENS, BROILERS',
                'CHICKENS, PULLETS, REPLACEMENT',
                'CHICKENS, ROOSTERS',
                'POULTRY, OTHER',
                'ALPACAS',
                'BISON',
                'EQUINE, HORSES & PONIES',
                'EQUINE, MULES & BURROS & DONKEYS',
                'GOATS',
                'HOGS',
                'LLAMAS',
                'RABBITS, LIVE',
                'SHEEP, INCL LAMBS',
                'CHUKARS',
                'DUCKS',
                'GEESE',
                'GUINEAS',
                'PEAFOWL, HENS & COCKS',
                'PIGEONS & SQUAB',
                'POULTRY, OTHER',
                'TURKEYS')


survey_livestock = survey %>% # PUT SURVEY INTO SHORT LIVESTOCK DATAFRAME
  filter(CROP_DESC %in% animal_list, head == TRUE, !grepl('LOSS, DEATH',HARVEST_DESC)) %>% 
  group_by(Program, Year,CROP_DESC) %>% 
  summarize(Value = max(Value))

census_livestock = census_animals %>% # PUT CENSUS INTO SHORT LIVESTOCK DATAFRAME
  filter(CROP_DESC %in% animal_list, HARVEST_DESC == 'INVENTORY') %>% 
  # !HARVEST_DESC %in% c(
  #   'SALES, MEASURED IN PCT OF FARM SALES',	'SALES, MEASURED IN $'),
  #   !grepl('OPERATIONS',HARVEST_DESC)) %>%
  group_by(Program, Year,CROP_DESC) %>% 
  summarize(Value = max(Value))

livestock_combined = census_livestock %>% # COMBINE AND CLEAN UP
  rbind(survey_livestock) %>% 
  spread(Program, Value) %>% 
  mutate(Value = if_else(is.na(CENSUS),SURVEY,CENSUS), Program = if_else(is.na(CENSUS),'SURVEY','CENSUS')) %>% 
  group_by(Year,CROP_DESC, Program) %>% 
  summarize(Value)


list = unique(livestock_combined$CROP_DESC)
# Backfill years
backfill = data.frame()
for (i in 1:length(list)){
  temp = data.frame(Year = 1990:2021, 
                    CROP_DESC = list[i], 
                    Value = 0, # Populate with zeros as placeholders
                    Program = 'temp')
  
  backfill = rbind(backfill,temp)
}

livestock_temp = rbind(livestock_combined, backfill) %>% 
  group_by(Year,CROP_DESC) %>% 
  summarize(Program = min(Program), Value = max(Value)) %>% 
  mutate(Value = ifelse(Value == 0, NA,Value),
         Program = ifelse(Program == 'temp','INTERPOLATED',Program))

starter = livestock_temp %>% 
  filter(!is.na(Value)) %>% 
  group_by(CROP_DESC) %>% 
  summarize(Year = min(Year))

ender = livestock_temp %>% 
  filter(!is.na(Value)) %>% 
  group_by(CROP_DESC) %>% 
  summarize(Year = max(Year))

first_value = starter %>% 
  left_join(livestock_temp) %>% 
  mutate(Year = min(livestock_temp$Year) - 1)

last_value = ender %>% 
  left_join(livestock_temp) %>% 
  mutate(Year = max(livestock_temp$Year) + 1)

livestock_full = first_value %>% 
  rbind(livestock_temp) %>% 
  rbind(last_value)

livestock = livestock_full %>% 
  group_by(CROP_DESC) %>% 
  mutate(head = na.approx(Value)) %>% 
  filter(Year >= min(livestock_temp$Year), Year < max(livestock_temp$Year)) %>% 
  select(-Value)


############ MERGE LIVESTOCK + MANURE MANAGEMENT ############ 

livestock_exact_ready = livestock %>% 
  rename(Livestock = CROP_DESC) %>% 
  left_join(livestock_mgmt, by = c('Year','Livestock')) %>% 
  mutate(`EX-ACT` = if_else(is.na(`EX-ACT`),"Chicken broiler",`EX-ACT`),
         Productivity = 'High-productivity',
         mgmt_pct = if_else(is.na(mgmt_pct),1,mgmt_pct),
         EF = if_else(is.na(EF),"",EF),
         PRP_pct = if_else(is.na(PRP_pct),"",PRP_pct),
         PRP_emissions = if_else(is.na(PRP_emissions),"",PRP_emissions),
         manure_mgmt = if_else(is.na(manure_mgmt),"Solid storage",manure_mgmt),
         `tCO2-e` = if_else(is.na(`tCO2-e` ),"",`tCO2-e`),
         head = head * mgmt_pct
  ) %>% 
  select(Year, Livestock, `EX-ACT`, mgmt_pct, Productivity, head, EF,PRP_pct,PRP_emissions, manure_mgmt,`tCO2-e`)


# SAVE IT
livestock_exact_ready %>% 
  write_csv("data/output/livestock_final_for_exact.csv")


total_cows = livestock_exact_ready %>% 
  filter(Livestock %in% c('CATTLE, COWS, MILK',
                          'CATTLE, (EXCL COWS)',
                          'CATTLE, COWS, BEEF'),
         Year == 2020) %>% 
  group_by() %>% 
  summarise(head = sum(head))

livestock_exact_ready %>% 
  filter(Livestock %in% c('CATTLE, COWS, MILK'),
         # 'CATTLE, (EXCL COWS)',
         # 'CATTLE, COWS, BEEF'),
         Year == 2020) %>% 
  group_by(manure_mgmt) %>% 
  summarise(head = sum(head), pct = sum(mgmt_pct)) %>% 
  mutate(pct = head/total_cows$head)



#############################################################################
########## PLOTS ########################################################
#############################################################################


livestock %>% 
  mutate(Source = Program) %>% 
  ggplot(aes(Year,head,color = fct_reorder2(CROP_DESC, Year, head))) +
  geom_line()+
  geom_point(aes(shape=Source),size = 2) +
  scale_fill_viridis(discrete = TRUE) +
  scale_y_continuous(labels = comma) +
  labs(color = "Livestock Description") +
  xlab("") +
  ylab("Number of Head") +
  theme_classic() +
  ggtitle("Livestock Head",subtitle = "Using USDA Census, Survey and Interpolated values")
ggsave("plots/livestock_head.png",width = 15, height = 6)

# ggplotly()

livestock %>% 
  filter(CROP_DESC %in% c(
    'CATTLE, COWS, MILK',
    'CATTLE, (EXCL COWS)',
    'CATTLE, COWS, BEEF',
    'CHICKENS, LAYERS'
  )) %>% 
  mutate(Source = Program) %>% 
  ggplot(aes(Year,head,color = fct_reorder2(CROP_DESC, Year, head))) +
  geom_line(size = 1.5,alpha = .8)+
  geom_point(aes(shape=Source),size = 3) +
  scale_color_viridis(discrete = TRUE) +
  scale_y_continuous(labels = comma) +
  labs(color = "Livestock") +
  xlab("") +
  ylab("Head of livestock") +
  theme_classic()
# ggtitle("Livestock High Head Count",subtitle = "Using USDA Census, Survey and Interpolated values")
ggsave("plots/livestock_common_head.png",width = 10, height = 5)

unique(livestock$CROP_DESC)

livestock %>% 
  filter(CROP_DESC %in% c(
    "CHICKENS, BROILERS","CHICKENS, PULLETS, REPLACEMENT",
    "CHICKENS, ROOSTERS","CHUKARS","DUCKS","GEESE","GUINEAS","PEAFOWL, HENS & COCKS", 
    "PIGEONS & SQUAB","POULTRY, OTHER","TURKEYS"
  )) %>% 
  mutate(Source = Program) %>% 
  ggplot(aes(Year,head,color = fct_reorder2(CROP_DESC, Year, head))) +
  geom_line(size = 1.5,alpha = .8)+
  geom_point(aes(shape=Source),size = 3) +
  scale_color_viridis(discrete = TRUE) +
  scale_y_continuous(labels = comma) +
  labs(color = "Other Poultry") +
  xlab("") +
  ylab("Head of poultry") +
  theme_classic()
# ggtitle("Livestock Low Head Count",subtitle = "Using USDA Census, Survey and Interpolated values")
ggsave("plots/livestock_other_poultry_head.png",width = 10, height = 5)

livestock %>% 
  filter(!CROP_DESC %in% c(
    'CHICKENS, LAYERS',"CHICKENS, BROILERS","CHICKENS, PULLETS, REPLACEMENT",
    "CHICKENS, ROOSTERS","CHUKARS","DUCKS","GEESE","GUINEAS","PEAFOWL, HENS & COCKS", 
    "PIGEONS & SQUAB","POULTRY, OTHER","TURKEYS",
    'CATTLE, COWS, MILK',
    'CATTLE, (EXCL COWS)',
    'CATTLE, COWS, BEEF'
  )) %>% 
  mutate(Source = Program) %>% 
  ggplot(aes(Year,head,color = fct_reorder2(CROP_DESC, Year, head))) +
  geom_line(size = 1.5,alpha = .8)+
  geom_point(aes(shape=Source),size = 3) +
  scale_color_viridis(discrete = TRUE) +
  scale_y_continuous(labels = comma) +
  labs(color = "Other Livestock") +
  xlab("") +
  ylab("Head of other livestock") +
  theme_classic()
# ggtitle("Livestock Low Head Count",subtitle = "Using USDA Census, Survey and Interpolated values")
ggsave("plots/livestock_other_head.png",width = 10, height = 5)

